%% DCBS function
% R.S., 2012
function [DCBS] = Compute_DCBS(cds)

% (input) cds: cell array of coding sequences
% (output) DCBS: numeric vector, same size as cds

bp_idx = [1 2 3; 1 2 6; 1 2 9; 1 2 12; 1 5 3; 1 5 6; 1 5 9; 1 5 12;
                             1 8 3; 1 8 6; 1 8 9; 1 8 12; 1 11 3; 1 11 6; 1 11 9; 1 11 12;
                             4 2 3; 4 2 6; 4 2 9; 4 2 12; 4 5 3; 4 5 6; 4 5 9; 4 5 12;
                             4 8 3; 4 8 6; 4 8 9; 4 8 12; 4 11 3; 4 11 6; 4 11 9; 4 11 12;
                             7 2 3; 7 2 6; 7 2 9; 7 2 12; 7 5 3; 7 5 6; 7 5 9; 7 5 12;
                             7 8 3; 7 8 6; 7 8 9; 7 8 12; 7 11 3; 7 11 6; 7 11 9; 7 11 12;
                             10 2 3; 10 2 6; 10 2 9; 10 2 12; 10 5 3; 10 5 6; 10 5 9; 10 5 12;
                             10 8 3; 10 8 6; 10 8 9; 10 8 12; 10 11 3; 10 11 6; 10 11 9; 10 11 12];


LC = length(cds);
f_xyz = zeros(64,LC);
f_bp = zeros(64,3*LC);
L = zeros(LC,1);
DCBS = zeros(LC,1);

for i = 1:LC
    
    seq = cds{i,1};
    L(i) = length(seq)/3;

    [numeric_seq, codons_counts,bp_counts] = translateStringToCodons(seq); 
    IID_freq_bp = zeros(64,3);
    codons_counts = codons_counts/L(i); %counts --> frequency
    bp_counts = bp_counts/L(i);
    f_xyz(:,i) = codons_counts;

   for k = 1:12
       [x,y] = find(bp_idx==k);
       IID_freq_bp(x,y) = bp_counts(k); 
   end

    f_bp(:,3*i-2:3*i) =  IID_freq_bp;
    IID_freq_codon = prod(IID_freq_bp,2);
    
    sel = codons_counts(numeric_seq)./IID_freq_codon(numeric_seq) ;
    anti_sel = IID_freq_codon(numeric_seq)  ./codons_counts(numeric_seq);

    rcb_xyz = max(sel, anti_sel) ; 
    s = sum (rcb_xyz);
    DCBS(i) = s/L(i);
   
end

end

function [numeric_seq,codons_counts,bp_counts] = translateStringToCodons(str)

    numeric_seq = [];
    sequence_index = 1;
    codons_counts = zeros(64,1);
    bp_counts = zeros(4,3);
    
    for i = 1:3:length(str)
        weight1 =  getNucleotidWeight(str(i));
        weight2 = getNucleotidWeight(str(i+1));
        weight3 = getNucleotidWeight(str(i+2));
        
        bp_counts(weight1,1) = bp_counts(weight1,1) + 1;
        bp_counts(weight2,2) = bp_counts(weight2,2) + 1;
        bp_counts(weight3,3) = bp_counts(weight3,3) + 1;

        codon_index = (weight1-1)*16 + (weight2-1)*4 + weight3;  
        numeric_seq(sequence_index) = codon_index;
        codons_counts(codon_index)= codons_counts(codon_index) +1; 
        sequence_index  = sequence_index + 1;
    end
    
        bp_counts = bp_counts';
        bp_counts = bp_counts(:);
end

function [weight] = getNucleotidWeight(letter)
    weight = 0;
    switch (letter)
        case {'A','a'}
            weight = 3; 
        case {'C','c'}
            weight = 2; 
        case {'G','g'} 
            weight = 4;
        case {'T', 'U','t','u'} 
            weight = 1; 
    end
end