% ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ %
%% This script loads and reads chloroplasts' genebanks, and creates the raw data table
% run this script from the directory of the downloaded genebanks
% ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ %

%% Read all files from dierctory of downloaded genebanks
a = dir('*.gb');
N = length(a);
genome_names={};
for i = 1:N
    warning('');
    Genome = genbankread(a(i).name);
       [warnmsg, msgid] = lastwarn;
    if strcmp(warnmsg,'The record references GenBank contigs. Sequence information is not available in this record.')
        continue
    end
    genome_names{1,i}=Genome.Source;
    genes=cell(length(Genome),1);
    UTR5=cell(length(Genome),1);
    UTR3=cell(length(Genome),1);
    ORF=cell(length(Genome),1);
    strands=zeros(length(Genome),1);
    chromseq=upper(Genome.Sequence);
    % remove empty genebank details:
    details=[Genome.CDS];
    if isempty(details)
        continue
    end
    ind={details.indices};
    ID={details.protein_id};
    names={details.gene};
    locus={details.text};
    ifcomplement={details.location};
    iteration=0;
    DATA={};
    locus_tag_genome={};
    strand=zeros(length(ind),1);
    % remove preblematic genes:
    remove_genes = [];
    for j=1:length(ind)
        if any(ind{j}<5) || any(ind{j}>length(chromseq)-5)
            remove_genes = [remove_genes,j];
            continue
        end
        if contains(ifcomplement{j},'join(complement(') % not a good indices for gene, ignore this gene
            remove_genes = [remove_genes,j];
            continue
        end
        if contains(ifcomplement{j},'complement') 
            strand(j)=-1;
            if sort(ind{j})~=fliplr(ind{j})  % it means the order isn't from min to max index, ignore this gene
                remove_genes = [remove_genes,j];
                continue
            end
        else
            strand(j)=1;
        end
    end
    new_j=setdiff(1:length(ind),remove_genes);
    ind(remove_genes)=[];
    ID(remove_genes)=[];
    names(remove_genes)=[];
    locus(remove_genes)=[];
    strand=strand(new_j);
    
    ordered_genes=zeros(length(ind)*2,1);
    for j=1:2:length(ind)*2
       ordered_genes(j)=min(ind{(j+1)/2});
       ordered_genes(j+1)=max(ind{(j+1)/2});
    end
    for j=1:length(ind)
        diff_ind1=ordered_genes-ordered_genes(j*2-1);
        diff_ind2=ordered_genes-ordered_genes(j*2);
        if strand(j)==1
            if ordered_genes(j*2-1)==min(ordered_genes)
                prev_gene=ordered_genes(j*2-1);
            else
                diff_prev=diff_ind1;
                diff_prev(diff_ind1>0)=inf;
                diff_prev(diff_ind1==0)=inf;
                prev_gene=min(abs(diff_prev));
            end
            
            if ordered_genes(j*2)==max(ordered_genes)
                next_gene=ordered_genes(j*2)-length(chromseq);
            else
                diff_next=diff_ind2;
                diff_next(diff_ind2<0)=inf;
                diff_next(diff_ind2==0)=inf;
                next_gene=min(diff_next);
            end
        else
            if ordered_genes(j*2)==max(ordered_genes)
                prev_gene=ordered_genes(j*2)-length(chromseq);
            else
                diff_prev=diff_ind2;
                diff_prev(diff_ind2<0)=inf;
                diff_prev(diff_ind2==0)=inf;
                prev_gene=min(diff_prev);
            end
            
            if ordered_genes(j*2-1)==min(ordered_genes)
                next_gene=ordered_genes(j*2-1);
            else
                diff_next=diff_ind1;
                diff_next(diff_ind1>0)=inf;
                diff_next(diff_ind1==0)=inf;
                next_gene=min(abs(diff_next));
            end
        end
        indices=ind{j};
        [utr5,orf,utr3,gene]=gene_seq(indices,strand(j),prev_gene,next_gene,chromseq);
        % remove genes that does not start with start codon and doeas
        % not end with stop codon:
        if ~strcmp(orf(1:3),'ATG')
            continue
        end
        if (sum(strcmp(orf(end-2:end),'TAG') + strcmp(orf(end-2:end),'TAA') + strcmp(orf(end-2:end),'TGA')))==0
            continue
        end
        % remove genes that are not deivisible by 3:
        if mod(length(orf),3)~=0
            continue
        end
        % take out genes with characters not nucleotides"
        if ~isempty(regexp(orf,'[^AaGgTtUuCc*]'))
            continue
        end
        % remove genes that the 5UTR is shorter than 10 nt:
        if length(utr5)<10
            continue
        end
        row=[];
        locus_tag=string(locus{j});
        for row=1:length(locus_tag)
            if ~isempty(regexp(locus_tag(row,1),'/locus_tag'))
                index=row;
                break
            end
        end
        str_locus_tag=locus_tag{row,1};
        locus_name=erase(str_locus_tag(12:end),' ');
        locus_name=erase(locus_name,'"');
        iteration=iteration+1;
        DATA{iteration,1}=gene;
        DATA{iteration,2}=orf;
        DATA{iteration,3}=utr5;
        DATA{iteration,4}=utr3;
        DATA{iteration,5}=ID{j};
        DATA{iteration,6}=names{j};
        DATA{iteration,7}=strand(j);
        DATA{iteration,8}=ind{j};
        DATA{iteration,9}=genome_names{1,i};
        DATA{iteration,10}=locus_name;
    end
    chloro=cell2table(DATA,'VariableNames',{'genes','ORF','UTR5','UTR3','protein_ID','genes_name','strand','ORF_indices','genomes_name','locus_tag'});
    % Don't save chloroplasts with an empty genebank information:
    if ~isempty(chloro)
        save(fprintf('chloro%d',iteration),'chloro')
    end
end

%% 16S seq
[~, Seq_16S] = get_rrn16('D:\9.11 study\study\study\master\thesis\chloroplasts\organisms_new\gene_banks_after_orgenizing');
save('Seq_16S','Seq_16S')

%% create binary T/F matrix for the energies according to position for all chloroplasts
% if a certain is shorter than 50 nt the T_F_mat will be filled with 0
% where there are no nt:
number_chloro = length(cloro_DATA);
save('number_chloro','number_chloro')
T_F_mat = cell(number_chloro,1);
for c=1:number_chloro
    c_mat=ones(length(cloro_DATA{c}.UTR5),50);
    count_pos=0;
    for pos=50:-1:1
        count_pos=count_pos+1;
        for g=1:length(cloro_DATA{c}.UTR5)
            if length(cloro_DATA{c}.UTR5{g})<pos
               c_mat(g,count_pos)=0; 
            end
        end
    end
    T_F_mat{c}=c_mat;
end
save('T_F_mat','T_F_mat')


%% calculate CAI
% calculate DCBS score for the reference set
DCBS_chloro=cell(number_chloro,1);
for c=1:number_chloro
    load(sprintf('chloro%d.mat',c))
    cur_DCBS=Compute_DCBS(chloro.ORF);
    DCBS_chloro{c}=cur_DCBS;
end

percent_highly_expressed = 0.3;
reference_chloro=cell(number_chloro,1);
for c=1:number_chloro
    if ~isempty(DCBS_chloro{c})
        [sorted,ind_s]=sort(DCBS_chloro{c},'Descend');
        percent=percent_highly_expressed*length(ind_s);
        reference_chloro{c}=ind_s(1:ceil(percent));
    end
end

% calculate CAI score according to the reference set
CAI_chloro=cell(number_chloro,1);
for c=1:number_chloro
    load(sprintf('chloro%d.mat',c))
    CAI=calculate_CAI(chloro.ORF, chloro.ORF(reference_chloro{c}));
    CAI_chloro{c}=CAI;
end

% calculate Z-scored CAI score
Z_score_CAI_chloro = cell(number_chloro,1);
for c=1:number_chloro
    Z_score_CAI_chloro{c} = (CAI_chloro{c}{c}-mean(CAI_chloro{c}{c}))/std(CAI_chloro{c}{c});
end
 
%% Add CAI and CAI Zscored to every gene in every chloroplast

for c = 1:num_chloro
    load(fprintf('chloro%d',iteration))
    chloro.CAI=CAI_chloro{c};
    chloro.CAI_Zscore=Z_score_CAI_chloro{c};
    chloro.DCBS = DCBS_chloro{c};
    save(fprintf('chloro%d',iteration),'chloro')
end