% simple function for finding idx of cells containing a specific substring

function index_vector =...
    find_cells_with_substring(cell_of_strings,wanted_substring)

%     take a cell of strings (N*1), search for cells with a specific substring
%     and return their indexes in a N*1 column vector

%     if substring is a cell of strings - find indexes for all substrings together
%     in this case if a cell contains more than 1 substring, show it only
%     once. 
%     also - return the vector sorted, from small to large

if ischar(wanted_substring)
    
    index_vector=cellfun(@(x) strfind(x,wanted_substring),...
        cell_of_strings,'UniformOutput',0);
    index_vector=find(cellfun(@isempty,index_vector)==0);

elseif iscellstr(wanted_substring)
    
    N=length(cell_of_strings); %get number of maximum finds
    M=length(wanted_substring); %get number of substrings
    index_vector=zeros(N,M); %create matrix with N*M 
    index_vector(:)=nan;     %fill it with nans
    
    for i=1:M   %loop to get indexes for each substring
    logical_M=cellfun(@(x) strfind(x,wanted_substring{i}),...
        cell_of_strings,'UniformOutput',0);
    logical_M=find(cellfun(@isempty,logical_M)==0);
    index_vector(1:(length(logical_M)),i)=logical_M;
    end
  
    %now reshape the matrix into a column vector
    %sort from small to large
    %get rid of repetitions
    
    index_vector=reshape(index_vector,[M*N,1]); %reshape
    index_vector(isnan(index_vector))=[];      %erase nans
    index_vector=unique(index_vector);         %delete repetitions + sort   
    
else
    error('secod input must be either a string or a cell of strings')

end

end