function [Org, Seq] = get_rrn16(Folder)
% [Org, Seq] = get_rrn16(Folder)
%
% go over all .gb files in a folder and get their rrn16 gene seqeunce
% if there is no such gene, leave the seq slot empty


cd(Folder)
a = dir('*.gb');
N = length(a);

% define outputs
Org = cell(N,1);
Seq = cell(N,1);

% get targets
Targets = {'rrn16';'16S'};

for i = 1:N
    warning('');
    RAW_data{i} = genbankread(a(i).name);
       [warnmsg, msgid] = lastwarn;
    if strcmp(warnmsg,'The record references GenBank contigs. Sequence information is not available in this record.')
        continue
    end
    i
    
    Org{i} = strrep(a(i).name,'.gb','');
    
    % get data
    Raw = genbankread(a(i).name);
    Genome = Raw.Sequence;
    Feat = featureparse(Raw.Features);
    try
        rRNA = Feat.rRNA;
    catch
        Seq{i} = nan;
        continue
    end
    
    % Get Querries
    if sum(strcmpi(fieldnames(rRNA), 'gene'))
        Names = get_cell_out_of_struct(rRNA,'gene');
    else
        Names = {};
    end
    if sum(strcmpi(fieldnames(rRNA), 'product'))
        Products = get_cell_out_of_struct(rRNA,'product');
    else
        Products = {};
    end
    if sum(strcmpi(fieldnames(rRNA), 'note'))
        Notes = get_cell_out_of_struct(rRNA,'note');
    else
        Notes = {};
    end
    
    % Search Targets in querry
    Fields = fieldnames(rRNA);
    
    I1 = find_cells_with_substring(Names,Targets);
    I2 = find_cells_with_substring(Products,Targets);
    I3 = find_cells_with_substring(Notes,Targets);
    
    Index = unique(vertcat(I1,I2,I3));
    switch length(Index)
        case 0
            Seq{i} = nan;
        case 1
            Indices = rRNA(Index).Indices;
            Seq{i} = extract_seq_by_cordinates(Genome, Indices);
        otherwise
            NN = length(Index);
            Options = cell(NN,1);
            for j = 1:NN
                Indices = rRNA(Index(j)).Indices;
                Options{j} = extract_seq_by_cordinates(Genome, Indices);
            end
            Seq{i} = Options;
    end

    Fname = [Folder '\16s_RNA.mat'];
    save(Fname, 'Seq', 'Org');
end

end