function group = calculate_graph_by_blast_for_group(gr)

load(sprintf('group%d.mat',gr)); 
%% initialization   
nodes_genes = [];
scores = [];
percents = [];
Evals = [];
ind =  1:height(group);
%% open scratch folder

%% create Headers and Sequences for BLAST
for gene = 1:height(group)
    ind = setdiff(ind,gene);
    cur_gene_name = {num2str(gene)};
    cur_gene_sequence = nt2aa(group.ORF(gene));
    Headers_genes=cell(length(ind),1);
    for gene_i = gene+1:height(group)
        Headers_genes{gene_i-gene}=num2str(gene_i);
    end
    Sequences_c=group.AA_ORF(ind);
%~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~%    
    %% BLAST
    % create fasta
    fastawrite(sprintf('gene%d_%d.fa',gene,gr), cur_gene_name, cur_gene_sequence)
    [~,cmdout]=system(sprintf('makeblastdb -in gene%d_%d.fa -dbtype prot',gene,gr));
    fastawrite(sprintf('genes%d_%d_query.fa',gene,gr), Headers_genes, Sequences_c)

    % blast
    [~,blast_gene] = system(sprintf('blastall -p blastp -d gene%d_%d.fa -i genes%d_%d_query.fa -m 8 -o gene%d_%d_blast.xml',gene,gr,gene,gr,gene,gr));

    % read blast
    blast_read = blastreadlocal(sprintf('gene%d_%d_blast.xml',gene,gr),8);

%~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~%    
    %% get the blast score for every couple of genes
    nodes_gene = ones(length(blast_read),2).*gene; 
    score = ones(length(blast_read),1);
    percent = ones(length(blast_read),1);
    Eval = ones(length(blast_read),1);
    for b = 1:length(blast_read)        
        nodes_gene(b,1)=str2num(blast_read(b).Query);
        score(b,1)=blast_read(b).Hits.HSPs(1).Score;
        percent(b,1)=blast_read(b).Hits.HSPs(1).Identities.Percent;
        Eval(b,1)=blast_read(b).Hits.HSPs(1).Expect;
    end
    
    scores=[scores;score];
    percents=[percents;percent];
    nodes_genes=[nodes_genes;nodes_gene];
    Evals=[Evals;Eval];
    save(sprintf('scores_%d',gr),'scores')
    save(sprintf('percents_%d',gr),'percents')
    save(sprintf('nodes_genes_%d',gr),'nodes_genes')
    save(sprintf('Evals_%d',gr),'Evals')
    save(sprintf('gene_%d',gr),'gene')
    save(sprintf('ind_%d',gr),'ind')
end
%~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~%    
%% create graph
EdgeTable = table(nodes_genes,Evals,'VariableNames',{'EndNodes' 'Weight'});
save(sprintf('EdgeTable_%d',gr),'EdgeTable')
%~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~%
%% Alighnment between the genes in the group
names=cell(height(group),1);
for gene =1:height(group)
    names{gene}=num2str(gene);
end
sequences=group.AA_ORF;
fastawrite(sprintf('gr%d.fa',gr), names, sequences)
[~,ali] = system(sprintf('clustalo  -i gr%d.fa  -o gr%d_ali.fa --outfmt=fasta --auto',gr,gr));

%% read Alignment and save
ali=fastaread(sprintf('gr%d_ali.fa',gr));
save(sprintf('gr%d_ali_read.mat',gr),'ali');

%% Eliminating genes that does not belong to the group:
%% First condition: Take pairs of genes that are similar enough
EdgeTable.Percent=percents;
EdgeTable.Score=scores;
Details=EdgeTable;
D=Details(find(Details.Percent>50 & Details.Weight<=10^-5),:);
EdgeTable_cur = table(D.EndNodes,D.Percent, 'VariableNames',{'EndNodes' 'Weight'});

%% Second condition: Eliminate genes with not enough arches
threshold_percent_arches = 0.40;
Graph=graph(EdgeTable_cur);
num_edges=degree(Graph);
low_arch=find(num_edges<(length(num_edges)-1)*threshold_percent_arches);
edges = num_edges(low_arch);
eliminate_genes_with_low_arch = low_arch;

%% Third condition: Elimiante genes that does not match the median length of the genes in the group
% calculate length of all genes in the group
l=zeros(height(group),1);
for gene=1:height(group)
    l(gene)=length(group.AA_ORF{gene});
end
mean_length = mean(l);
eliminate_genes_by_length=[];
for gene = 1:height(group)
    if (100*(length(group.AA_ORF{gene})/mean_length)<=65 || 100*(length(group.AA_ORF{gene})/mean_length)>=140)
       eliminate_genes_by_length = [eliminate_genes_by_length;gene];
    end
end

%% Save the filtered group
eliminate_genes = unique([eliminate_genes_by_length;eliminate_genes_with_low_arch]);
save_genes = setdiff(1:height(group),eliminate_genes);
group = group(save_genes,:);
end
